#ifndef GDATA_UPLOADER_H
#define GDATA_UPLOADER_H

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>

#include <gdata/gdata-service.h>
#include <gdata/gdata-entry.h>

G_BEGIN_DECLS

/**
 * GDATA_LINK_RESUMABLE_CREATE_MEDIA:
 *
 * The relation type URI of the resumable upload location for resources attached to this resource.
 *
 * For more information, see the
 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/resumable_upload.html#ResumableUploadInitiate">GData resumable upload protocol
 * specification</ulink>.
 *
 * Since: 0.13.0
 */
#define GDATA_LINK_RESUMABLE_CREATE_MEDIA "http://schemas.google.com/g/2005#resumable-create-media"

/**
 * GDATA_LINK_RESUMABLE_EDIT_MEDIA:
 *
 * The relation type URI of the resumable update location for resources attached to this resource.
 *
 * For more information, see the
 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/resumable_upload.html#ResumableUploadInitiate">GData resumable upload protocol
 * specification</ulink>.
 *
 * Since: 0.13.0
 */
#define GDATA_LINK_RESUMABLE_EDIT_MEDIA "http://schemas.google.com/g/2005#resumable-edit-media"

#define GDATA_TYPE_UPLOADER		(gdata_uploader_get_type ())

G_DECLARE_FINAL_TYPE (GDataUploader, gdata_uploader, GDATA, UPLOADER, GObject);

typedef struct _GDataUploaderPrivate	GDataUploaderPrivate;

struct _GDataUploader {
	GObject parent;
	GDataUploaderPrivate *priv;
};

struct _GDataUploaderClass {
	/*< private >*/
	GObjectClass parent;

	/*< private >*/
	/* Padding for future expansion */
	void (*_g_reserved0) (void);
	void (*_g_reserved1) (void);
	void (*_g_reserved2) (void);
	void (*_g_reserved3) (void);
	void (*_g_reserved4) (void);
	void (*_g_reserved5) (void);
};

GDataUploader *gdata_uploader_new (GDataService *service, GDataAuthorizationDomain *domain, const gchar *method, const gchar *upload_uri,
                                        GDataEntry *entry, const gchar *slug, const gchar *content_type) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
GDataUploader *gdata_uploader_new_resumable (GDataService *service, GDataAuthorizationDomain *domain, const gchar *method, const gchar *upload_uri,
                                                  GDataEntry *entry, const gchar *slug, const gchar *content_type, goffset content_length) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;

void gdata_uploader_set_input (GDataUploader *self, GInputStream *stream);
void gdata_uploader_set_input_from_bytes (GDataUploader *self, GBytes *bytes);

GBytes *gdata_uploader_send (GDataUploader *self, GCancellable *cancellable, gsize *total_written, GError **error);
void gdata_uploader_send_async (GDataUploader *self, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data);
GBytes *gdata_uploader_send_finish (GDataUploader *self, GAsyncResult *result, gsize *total_written, GError **error);

GDataService *gdata_uploader_get_service (GDataUploader *self) G_GNUC_PURE;
GDataAuthorizationDomain *gdata_uploader_get_authorization_domain (GDataUploader *self) G_GNUC_PURE;
const gchar *gdata_uploader_get_method (GDataUploader *self) G_GNUC_PURE;
const gchar *gdata_uploader_get_upload_uri (GDataUploader *self) G_GNUC_PURE;
GDataEntry *gdata_uploader_get_entry (GDataUploader *self) G_GNUC_PURE;
const gchar *gdata_uploader_get_slug (GDataUploader *self) G_GNUC_PURE;
const gchar *gdata_uploader_get_content_type (GDataUploader *self) G_GNUC_PURE;
goffset gdata_uploader_get_content_length (GDataUploader *self) G_GNUC_PURE;

G_END_DECLS

#endif /* !GDATA_UPLOAD_STREAM_H */
